/**
 * @file main.cpp Danisauna 3a
 *
 * Esp8266 Wifi / SoftAP setup via Noextion UI
 *
 * @author Jyrki Berg 2/17/2019 (https://github.com/jyberg)
 * 
 * @copyright 
 * Copyright (C) 2019 Jyrki Berg \n
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */
#include <Arduino.h>
// for debug printing to Serial0, by default Serial uses Serial 1 (Swapped ports)
#define DEBUG
#include "debugPrint.h"

#include "globalSettings.h"
#include "wifi.h"
#include "nextionUi.h"
#include <HardwareSerial.h>
#include <time.h>
#include <pubsubclient.h>

String Version = "DaniSauna_1.1.2";
//HTTP Update
#define FirmwareVer  5 // Bitte hier Version der Firmware eintragen
#define URL_Mit_Version_Info "http://www...../httpUpdate.txt" 
#define URL_Mit_Firmware "http://www...../httpUpdate.bin"
#include <ESP8266HTTPClient.h>
#include <ESP8266httpUpdate.h>
HTTPClient http;

// Feuchte und Tempsensor für Sauna 
#include <CFF_ChipCap2.h>
CFF_ChipCap2 cc2 = CFF_ChipCap2();

// IIC-Portexpander
#include <pcf8574_esp.h>
TwoWire testWire;
PCF857x pcf8574(0x20, &testWire);

// AD-Wandler ADS1115
#include<ADS1115_WE.h> 
#define I2C_ADDRESS 0x48
ADS1115_WE adc = ADS1115_WE();

// RTC DS2321
#include "RTClib.h"
RTC_DS3231 rtc;
char daysOfTheWeek[7][12] = {"Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"};

// NTP Einbindung
#include <NTPtimeESP.h>
NTPtime NTPch("pool.ntp.org");
strDateTime dateTime;

using namespace nextionUi;
using namespace wifi;

uint32_t lastimeUpdate{0};
uint32_t timeUpdateInterval{1000};
uint32_t lastInfoUpdate{0};
uint32_t InfoUpdateInterval{5000};
uint8_t  prevPage{255};


float fTempIst;
float fDampfIst;
float fOfenIst;
int   iOfenSet;
int   iDampfSet;
int   i_Test;
float fTemp0;
float fTemp1;
float fTemp2;
float fTemp3;
int   Ueberhitzung;

char MQTT_Basis [18] = "DaniSauna/stat";

unsigned long intervallShowTemp = 9000; // in msec | alle 9 Sekunden Temp aktualisieren
unsigned long TimerShowTemp = (unsigned long) millis() + intervallShowTemp;
unsigned long intervallMessTemp = 4500; // in msec | alle 4,5 Sekunden nächsten Sensor Messen
unsigned long TimerMessTemp = (unsigned long) millis() + intervallMessTemp;
unsigned long intervallRTCset= 21600000; // in msec | nach  6 Stnden = 6+60+60+1000 Rtc aktualisieren
unsigned long TimerRTCset = millis();; 
unsigned long intervallPolling = 18000; // in msec | Regelungung
unsigned long TimerPolling;
unsigned long intervallTW = 10000; // in msec | 10000 jede 10 Sekunden Tür und Wasser Prüfen
unsigned long TimerTW;
unsigned long intervallTuerOffen = 60000; // in msec | nach 10Minuten Sauna aus weil Tür zu lange offen
unsigned long TimerTuerOffen;
unsigned long intervallMQTT = 29000; // in msec | 29000 jede 29 Sekunden MQTT-Senden
unsigned long TimerMQTT;

//Anderer Weg bei Befeuchter:
unsigned long BefeuchterZyklus = 4*60000; //AN-Aus Zyklus ist 4Min
unsigned long BefeuchterAnZeit = 3*60000; //An Zeit muss kleiner als Zykluszeit sein
unsigned long BefeuchterZeitmarke;

byte  iADS_Temp_Cnt = 0;

// MQTT
const int     MQTT_port = 1881;
WiFiClient    MQTT_espClient;
PubSubClient  MQTT_Client(MQTT_espClient);
IPAddress     MQTT_Server(192, 168, 178, 1);
static char   arTXT[40];
long          lastReconnectAttempt = 0;

// Init Variablen für MQTT
// Sollen auf MQTT-server zurückgespeichert Werden
// Problem wenn Ofen oder Dampf An und es wird ausgeschaltet. 
uint8_t Init_Dampf;
uint8_t Init_DampfSoll;
char 	Init_Einschalt[41];
uint8_t Init_Licht0; 
uint8_t Init_Licht1;
uint8_t Init_Licht2;
char 	Init_MaxDauer[41];
uint8_t Init_Ofen;
uint8_t Init_OfenSoll;
uint8_t Init_Termin;
unsigned long intervallInitMQTT = 10000; // in msec | 10000 jede 10 Sekunden Tür und Wasser Prüfen
unsigned long TimerInitMQTT;
uint8_t Init_MQTT = 0;

// State-Maschine
const uint8_t State_Aus = 0;
const uint8_t State_U_V_An = 1;
const uint8_t State_U_V_W_An = 2;
const uint8_t State_U_V_W1_An = 3;
const uint8_t State_W1_An = 4;
const uint8_t Phase_U_V = 4;
const uint8_t Phase_W = 5;
const uint8_t Phase_W1 = 6;
const uint8_t Ein = 0;
const uint8_t Aus = 1;
uint8_t Status = 0;
uint8_t Status_alt = 0;
uint8_t L0Status = 0;
uint8_t L1Status = 0;
uint8_t L2Status = 0;

// ***************************************************************************************
// Prüfen für FirmwareUpdate
// gefunden bei: https://esp8266-server.de/wemos.html
// ***************************************************************************************
void FirmwareUpdate()
{
  // Überprüfen der Firmwareversion des programmms aud dem Server
  int FirmwareNeu = 0;
  http.begin(URL_Mit_Version_Info);     // Webseite aufrufen
  int httpCode = http.GET();            // Antwort des Servers einlesen
  if (httpCode == HTTP_CODE_OK)         // Wenn Antwort OK
  {
    String payload = http.getString();  // Webseite einlesen
    FirmwareNeu = payload.toInt();      // Zahl aus Sting bilden
  } 
  http.end();
  
  if (FirmwareNeu > FirmwareVer)        // Firmwareversion mit aktueller vergleichen
  {
    Serial.println("Neue Firmware verfuegbar");
    Serial.println("Starte Download");
	kInfo += "Neue Firmware verfuegbar\\r";
	kInfo += "Starte Download\\r";
	StartKonsole(kInfo);
    ESPhttpUpdate.rebootOnUpdate(false);// reboot abschalten, wir wollen erst Meldungen ausgeben
    t_httpUpdate_return ret = ESPhttpUpdate.update(URL_Mit_Firmware);
    switch (ret)
    {
      case HTTP_UPDATE_FAILED:
        Serial.printf("HTTP_UPDATE_FAILD Error (%d): %s", ESPhttpUpdate.getLastError(), ESPhttpUpdate.getLastErrorString().c_str());
		kInfo += "HTTP_UPDATE_FAILD Error\\r";
		StartKonsole(kInfo);
        break;
      case HTTP_UPDATE_OK:
        Serial.println("Update erfolgreich");
        Serial.flush();
        Serial.println("Reset");
        Serial.flush();
     	kInfo += "Update erfolgreich\\r";
		kInfo += "Reset\\r";
		StartKonsole(kInfo);
   		delay(2000);
        ESP.reset();
        delay(100);
        break;
    }
  } 
}
// ***************************************************************************************
// Prüfen für MQTT-CallBack
// ***************************************************************************************
boolean Pruefe(char* tTopic, String Wert) {
  char arTopic[30] = "";
  String sTopic = MQTT_Basis;
  sTopic += "/";
  sTopic += Wert;
  sTopic.toCharArray(arTopic, sTopic.length() + 1);
  if (strcmp(tTopic, arTopic) == 0) {
    return true;
  } else {
    return false;
  }
}
// ***************************************************************************************
// MQTT-CallBack
// ***************************************************************************************
void MQTT_Callback(char* topic, byte* payload, unsigned int length) {
  String SPayload = "";
  boolean Speichern = false;
  if (Init_MQTT  < 10) 
	{
	   return;
	}//Nur beim ersten Mal die Initdaten senden
  
  Serial.print("Nachricht eingetroffen [");
  Serial.print(topic);
  Serial.print("]: ");
  
  for (unsigned int i = 0; i < length; i++)
  {
    SPayload += (char) payload[i];
    Serial.print((char) payload[i]);
  }
  Serial.println();

  // eingangs Topic Auswertung
  byte bTest;
  if (Pruefe(topic, "Dampf")) {
    if (SPayload != 0) {
      //Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
      //Serial.println(arTXT);
	  bTest = 1;
	  if (strcmp(arTXT, "true") != 0)
	  {
		  bTest = 0;
	  }
      if (bTest != Dampf) {
        Dampf = bTest;
		DampfNeu();
		Tastenwechsel = 1;
		Speichern = false;
      }
    }
  }

  if (Pruefe(topic, "Ofen")) {
    if (SPayload != 0) {
      //Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
      //Serial.println(arTXT);
	  bTest = 1;
	  if (strcmp(arTXT, "true") != 0)
	  {
		  bTest = 0;
	  }
      if (bTest != Ofen) {
        Ofen = bTest;
		OfenNeu();
		Tastenwechsel = 1;
		Speichern = true;
      }
    }
  }

  if (Pruefe(topic, "Termin")) {
    if (SPayload != 0) {
      //Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
      //Serial.println(arTXT);
	  bTest = 1;
	  if (strcmp(arTXT, "true") != 0)
	  {
		  bTest = 0;
	  }
      if (bTest != gSettings.Termin) {
        gSettings.Termin = bTest;
		Termin_alt = bTest;
		TerminNeu();
		Tastenwechsel = 1;
		Speichern = true;
      }
    }
  }

  if (Pruefe(topic, "Licht0")) {
    if (SPayload != 0) {
     // Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
     // Serial.println(arTXT);
	  bTest = 1;
	  if (strcmp(arTXT, "true") != 0)
	  {
		  bTest = 0;
	  }
      if (bTest != Licht0_alt) {
        gSettings.Licht0 = bTest;
		Lampe0Neu();
		Tastenwechsel = 1;
		Speichern = true;
      }
    }
  }

  if (Pruefe(topic, "Licht1")) {
    if (SPayload != 0) {
      //Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
      //Serial.println(arTXT);
	  bTest = 1;
	  if (strcmp(arTXT, "true") != 0)
	  {
		  bTest = 0;
	  }
      if (bTest != Licht1_alt) {
        gSettings.Licht1 = bTest;
		Lampe1Neu();
		Tastenwechsel = 1;
		Speichern = true;
      }
    }
  }

  if (Pruefe(topic, "Licht2")) {
    if (SPayload != 0) {
      //Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
      //Serial.println(arTXT);
	  bTest = 1;
	  if (strcmp(arTXT, "true") != 0)
	  {
		  bTest = 0;
	  }
      if (bTest != Licht2_alt) {
        gSettings.Licht2 = bTest;
		Lampe2Neu();
		Tastenwechsel = 1;
		Speichern = true;
      }
    }
  }

  if (Pruefe(topic, "DampfSoll")) {
    if (SPayload != 0) {
      //Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
      //Serial.println(arTXT);
	  bTest = atoi(arTXT);
      if ( bTest != DampfSoll_alt) {
        DampfSoll = bTest;
        DampfSoll_alt = bTest;
		gSettings.vDampfSollSet = bTest;
		DampfSollNeu();
		Tastenwechsel = 1;
		Speichern = true;
      }
    }
  }

  if (Pruefe(topic, "OfenSoll")) {
    if (SPayload != 0) {
      //Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
      //Serial.println(arTXT);
	  bTest = atoi(arTXT);
      if ( bTest != OfenSoll_alt) {
        OfenSoll = bTest;
        OfenSoll_alt = bTest;
		gSettings.vTempSollSet = bTest;
		OfenSollNeu();
		Tastenwechsel = 1;
		Speichern = true;
      }
    }
  }

  char puffer[3];
  uint8_t hh;
  uint8_t mm;
  uint8_t DD;
  uint8_t MM;
  if (Pruefe(topic, "MaxDauer")) {
    if (SPayload != 0) {
     // Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
	  puffer[0]=arTXT[0];
	  puffer[1]=arTXT[1];
	  hh = atoi (puffer);
	  puffer[0]=arTXT[3];
	  puffer[1]=arTXT[4];
	  mm = atoi (puffer);
	  if (gSettings.vhSet != hh || gSettings.vmSet != mm)
	  {
		  gSettings.vhSet = hh;
		  gSettings.vmSet = mm;
		  LaufzeitNeu();
		  Speichern = true;
	  }
    }
  }	

  if (Pruefe(topic, "Einschalt")) {
    if (SPayload != 0) {
      //Serial.println(SPayload);
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
	  puffer[0]=arTXT[0];
	  puffer[1]=arTXT[1];
	  hh = atoi (puffer);
	  puffer[0]=arTXT[3];
	  puffer[1]=arTXT[4];
	  mm = atoi (puffer);
	  puffer[0]=arTXT[6];
	  puffer[1]=arTXT[7];
	  DD = atoi (puffer);
	  puffer[0]=arTXT[9];
	  puffer[1]=arTXT[10];
	  MM = atoi (puffer);	
	  if (gSettings.vhh != hh || gSettings.vmm != mm ||  gSettings.vDD != DD || gSettings.vMM != MM)
	  {
		  gSettings.vhh = hh;
		  gSettings.vmm = mm;
		  gSettings.vDD = DD;
		  gSettings.vMM = MM;
	      StartZeitNeu();
		  Speichern = true;
	  }	    	
	}
  }

  if (Pruefe(topic, "Reboot")) {
    if (SPayload != 0) {
      SPayload.toCharArray(arTXT, SPayload.length() + 1);
 	  if (strcmp(arTXT, "false") != 0)
	  {
        Serial.println("Reset");
        Serial.flush();
   		delay(2000);
        ESP.reset();
        delay(100);
      }
    }
  }

  if (Speichern == true)
  {
	  StoreGlobalSettings();
	  Speichern = false;
  }
}
// ***************************************************************************************
// MQTT-Reconnect
// ***************************************************************************************
boolean MQTT_reconnect()
{
	if (MQTT_Client.connect("DaniSauna", "MQTT-user","Geheim"))
	{
		// Once connected, publish an announcement...
		MQTT_Client.publish("DaniSauna", "online");
		Version.toCharArray(arTXT, Version.length() + 1);
		MQTT_Client.publish("DaniSauna/Firmware", arTXT);
		// ... and resubscribe
		MQTT_Client.subscribe("DaniSauna/stat/#");
	}
	return MQTT_Client.connected();
}
// ***************************************************************************************
// PCF Initialisieren
// ***************************************************************************************
void PCFInit()
{
	//SDA = GPIO.12 (D6), SCL = GPIO.14 (D5)
	testWire.begin(12, 14); //testWire.begin(SDA, SCL)
	testWire.setClock(100000L);
	pcf8574.begin();
	if (pcf8574.lastError() != 0)
	{
		Serial.println("PCF nicht erreichbar !");
		InitFehler = true;
	}
	else
	{
		Serial.println("PCF ist OK !");
	}
	//PCF alle Lichter Aus 
	pcf8574.write(3,0 );
	pcf8574.write(2,0 );
	pcf8574.write(1,0 );
}
// ***************************************************************************************
// CC2 Initialisieren
// ***************************************************************************************
void CC2Init()
{
	cc2.begin();
	cc2.startCommandMode();
  	delay(100);
	if (cc2.status == CCF_CHIPCAP2_STATUS_COMMANDMODE)
	{
		cc2.startNormalMode();
		delay(100);
	}
	else
	{
		//DebugPrintLn("ChipCap2 is in normal mode."); 
	}
  	delay(2);
	if (cc2.status == 1)
	{
		kInfo +="CC2 OK\\r";
	}
	else
	{
		kInfo +="CC2 nicht erreichbar\\r";
	}
}
// ***************************************************************************************
// ADS Initialisieren
// ***************************************************************************************
void ADSInit()
{
  Wire.begin();
  if(!adc.init()){
    DebugPrintLn("ADS1115 not connected!");
	kInfo +="ADS1115 nicht erreichbat!\\r";
	abort();
  }
  adc.setVoltageRange_mV(ADS1115_RANGE_6144); //comment line/change parameter to change range
  adc.setCompareChannels(ADS1115_COMP_0_GND); //comment line/change parameter to change channel
  adc.setConvRate(ADS1115_860_SPS); //uncomment if you want to change the default
  adc.setMeasureMode(ADS1115_SINGLE); //comment line/change parameter to change mode
  DebugPrintLn("ADS1115 OK");
  kInfo +="ADS1115 OK\\r";
}
// ***************************************************************************************
// Uhrzeit zusammenbaun           
// ***************************************************************************************
String DatumZeit() 
{
	DateTime t = rtc.now();
	String Z = "";

	int x = t.hour();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	//Z += ":";
	Z += ":";
	x = t.minute();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += ":";
	x = t.second();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += " ";
	x = t.day();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += ".";
	x = t.month();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += ".";
	x = t.year();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	return Z;
}
String Datum() 
{
	DateTime t = rtc.now();
	String Z = "";

	int x = t.day();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += ".";
	x = t.month();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += ".";
	x = t.year();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	String Temp = String(x);
	Temp.toCharArray(gSettings.vYYYY, Temp.length() + 1);
	Jahr=x;
	return Z;
}
String Zeit() 
{
	DateTime t = rtc.now();
	String Z = "";
	int x = t.hour();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += ":";
	x = t.minute();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	//Z += ":";
	return Z;
}
String Zeits()
{
	DateTime t = rtc.now();
	String Z = "";
	int x = t.hour();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += ":";
	x = t.minute();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	Z += ":";
	x = t.second();
	if (x < 10) 
	{
		Z += "0";
	}
	Z += String(x);
	return Z;
}
// ***************************************************************************************
// RTC Initialisierung
// ***************************************************************************************
void InitRTC()
{
	if (! rtc.begin())
	{
		DebugPrintLn("Couldn't find RTC");
		kInfo +="RTC nicht erreichbar\\r";
		abort();
	}
	else
	{
		DebugPrintLn("RTC OK");
		kInfo +="RTC OK\\r";
	}
}
void RTC_Set()
{
	//Nur neu setzen wenn WiFi connected ist
	if (WiFi.status() == WL_CONNECTED) {
		dateTime = NTPch.getNTPtime(1.0, 1);
		setTime(dateTime.hour, dateTime.minute, dateTime.second,
				dateTime.day, dateTime.month, dateTime.year);
		rtc.adjust(DateTime(dateTime.year, dateTime.month, dateTime.day, dateTime.hour, dateTime.minute, dateTime.second));
		Serial.println(DatumZeit());
	}
}
// ***************************************************************************************
// Temps Messen
// CC2 nur alle 7 Sekunden Messen damit gültiges Ergebnis
// ***************************************************************************************
void MessTemps()
{
	if ((unsigned long) (millis() - TimerMessTemp) >= intervallMessTemp)
	{
		//Serial.println("MessTemps");
		TimerMessTemp = millis();
		switch (iADS_Temp_Cnt)
		{
		case 0:
			if (cc2.dataReady() == true)
			{
				cc2.readSensor();
				fTempIst = cc2.temperatureC;
				dtostrf(fTempIst, 2, 1, TempIst);
				//Serial.print("CC2 :");Serial.print(TempIst);Serial.println("°C");	
				fDampfIst = cc2.humidity;
				dtostrf(fDampfIst, 2, 1, DampfIst);
				//Serial.print("CC2 :");Serial.print(DampfIst);Serial.println("%");	
			}
			break;
		case 1:
			adc.setCompareChannels(ADS1115_COMP_0_GND);
			break;
		case 2:
			adc.setCompareChannels(ADS1115_COMP_1_GND);
			break;
		case 3:
			adc.setCompareChannels(ADS1115_COMP_2_GND);
			break;
		case 4:
			adc.setCompareChannels(ADS1115_COMP_3_GND);
			break;
		default:
			break;
		}
		switch (iADS_Temp_Cnt)
		{
		case 0:
			iADS_Temp_Cnt = iADS_Temp_Cnt + 1; //nur 1 Fühler überm Ofen 
			break;
		default:		
			if (not adc.isBusy())
			{
				adc.startSingleMeasurement();	
				#define SERIESRESISTOR 10000
				#define NOMINAL_RESISTANCE 10000
				#define NOMINAL_TEMPERATURE 25
				#define BCOEFFICIENT 3950	
				float ADCvalue;
				float Resistance;
				ADCvalue = adc.getResult_V(); //in V
				Resistance = SERIESRESISTOR * ADCvalue/(4.46-ADCvalue); //in Ohm

				float steinhart;
				steinhart = Resistance / NOMINAL_RESISTANCE; // (R/Ro)
				steinhart = log(steinhart); // ln(R/Ro)
				steinhart /= BCOEFFICIENT; // 1/B * ln(R/Ro)
				steinhart += 1.0 / (NOMINAL_TEMPERATURE + 273.15); // + (1/To)
				steinhart = 1.0 / steinhart; // Invert
				steinhart -= 273.15; // convert to C

				switch (iADS_Temp_Cnt)
				{
				case 1:
					fOfenIst = steinhart;
					fTemp0 = steinhart;
					dtostrf(fOfenIst, 2, 1, OfenIst);	
					//Serial.print("NTC0 :");Serial.print(OfenIst);Serial.println("°C");		
					break;
				case 2:
					fOfenIst = steinhart;
					fTemp1 = steinhart;
					//dtostrf(fOfenIst, 2, 1, OfenIst);			
					//Serial.print("NTC1 :");Serial.print(OfenIst);Serial.println("°C");		
					break;
				case 3:
					fOfenIst = steinhart;
					fTemp2 = steinhart;
					//dtostrf(fOfenIst, 2, 1, OfenIst);			
					//Serial.print("NTC2 :");Serial.print(OfenIst);Serial.println("°C");		
					break;
				case 4:
					fOfenIst = steinhart;
					fTemp3 = steinhart;
					//dtostrf(fOfenIst, 2, 1, OfenIst);			
					//Serial.print("NTC3 :");Serial.print(OfenIst);Serial.println("°C");		
					break;
				default:
					break;
				}
				if (iADS_Temp_Cnt <2) //Nur 1 NTCx zur verfügung damit ist die Schleifen zeit 8s/4
				{
					iADS_Temp_Cnt = iADS_Temp_Cnt + 1;
				}
				else
				{
					iADS_Temp_Cnt = 0;
				}
			}
			break;
		}
		//Serial.println("MessTemps Ende");
	}
}
// ***************************************************************************************
// PCF Schalten 1=Schalten auf Masse !!!!
// ***************************************************************************************
void Licht_Schalten()
{
	if (gSettings.Licht0 != Licht0_alt)
	{
		pcf8574.write(3,gSettings.Licht0 );
		Licht0_alt=gSettings.Licht0;
		L0Status = Licht0_alt;
	}
	if (gSettings.Licht1 != Licht1_alt)
	{	
		pcf8574.write(2,gSettings.Licht1 );
		Licht1_alt=gSettings.Licht1;
		L1Status = Licht1_alt;
	}	 
	if (gSettings.Licht2 != Licht2_alt)
	{
		pcf8574.write(1,gSettings.Licht2 ); 
		Licht2_alt=gSettings.Licht2;
		L2Status = Licht2_alt;
	}
}
// ***************************************************************************************
// PCF Schalten 1=Schalten auf Masse !!!!
// ***************************************************************************************
void Status_Schalten()
{
	if (Status != Status_alt)
	{
		switch (Status)
		{
		case State_U_V_An :
			pcf8574.write(Phase_U_V,Ein);
			pcf8574.write(Phase_W,Aus);
			pcf8574.write(Phase_W1,Aus);
			break;

		case State_U_V_W_An :
			pcf8574.write(Phase_U_V,Ein);
			pcf8574.write(Phase_W1,Aus);
			delay(50);
			pcf8574.write(Phase_W,Ein);
			break;

		case State_U_V_W1_An :
			pcf8574.write(Phase_U_V,Ein);
			pcf8574.write(Phase_W,Aus);
			delay(50);
			pcf8574.write(Phase_W1,Ein);
			break;

		case State_W1_An :
			pcf8574.write(Phase_U_V,Aus);
			pcf8574.write(Phase_W,Aus);
			delay(50);
			pcf8574.write(Phase_W1,Ein);
			break;
		default: //ALLES AUS
			pcf8574.write(Phase_U_V,Aus);
			pcf8574.write(Phase_W,Aus);
			pcf8574.write(Phase_W1,Aus);
			break;
		}
		Status_alt = Status;
	}
}
// ***************************************************************************************
// Tempregelung
// ***************************************************************************************
void TempRegelung()
{

/*
	Serial.print("Ofen = ");Serial.println(Ofen);
	Serial.print("Damof = ");Serial.println(Dampf);
	Serial.print("Temp = ");Serial.println(OfenSoll);
	Serial.print("Feucht = ");Serial.println(DampfSoll);
*/

	//Zuheiss alle aus
	if (fOfenIst >= 124) //Bei mehr als 123Grad ABSCHALTEN
	{
		Ofen  = 0;
		Dampf = 0;
		Ueberhitzung = 1;
		Status = State_Aus;
		return;
	}
	else
	{
		Ueberhitzung = 0;
	}
		
	//NormalBetrieb
	if (Ofen == 1)
	{
		if (OfenSoll > fTempIst)
		{
			Status = State_U_V_W_An;
			//Hier Hysterese einbauen
		}
		else
		{
			Status = State_Aus;
		}
	}
	else
	{
		//Alle ausschalten;
		Ofen  = 0;
		Dampf = 0;
		Status = State_Aus;	
	}
	
	//Verdampfer ist im Status bis jetzt aus;
	if (Dampf == 1) 
	{
		if (DampfSoll > fDampfIst )
		{
			//Hier Hysterese einbauen
			if (millis() - BefeuchterZeitmarke >= BefeuchterZyklus) 
			{
				BefeuchterZeitmarke = millis();
			}
			if (millis() - BefeuchterZeitmarke < BefeuchterAnZeit)
			{
				//Verdampfer einschalten	
				switch (Status)
				{
				case State_Aus:
					Status = State_W1_An;
					break;
				
				case State_U_V_An:
					Status = State_U_V_W1_An;
					break;
				
				case State_U_V_W_An:
					Status = State_U_V_W1_An;
					break;
				}
			}
		}
	}
}
// ***************************************************************************************
// Startwerte setzen
// ***************************************************************************************
void Werte_Initialisieren()
{
	//Alles lampen aus
	Licht0_alt = 111;
	Licht1_alt = 111;
	Licht2_alt = 111;
	TuerOffen_alt =111;
	WasserLeer_alt = 111;
	//Ofen und Verdampfer aus
	Ofen = 0;
	Dampf = 0;
	Status = State_Aus;
	//Temperaturen vom letzten Mal eintragen
	Ueberhitzung = 0;
	TuerOffen = 0;
	WasserLeer = 0;
	DampfSoll = gSettings.vDampfSollSet;
	OfenSoll = gSettings.vTempSollSet;
	Tastenwechsel = 1;
	prevPage = 111;

	//MQTT Initialisierungen
	Init_Dampf = 0;
	Init_DampfSoll = gSettings.vDampfSollSet;
	sprintf(Init_Einschalt, "%02d:%02d %02d.%02d", gSettings.vhh, gSettings.vmm,gSettings.vDD,gSettings.vMM);
	Init_Licht0 = gSettings.Licht0;
	Init_Licht1 = gSettings.Licht1;
	Init_Licht2 = gSettings.Licht2;
	sprintf(Init_MaxDauer, "%02d:%02d", gSettings.vhSet, gSettings.vmSet);
	Init_Ofen = 0;
	Init_OfenSoll = gSettings.vTempSollSet;
	Init_Termin = gSettings.Termin;
}
// ***************************************************************************************
// Tür und Wasser-Sensor
// ***************************************************************************************
void TuWSensor()
{
	if ((unsigned long) (millis() - TimerTW) >= intervallTW)
	{
		//Serial.println("TuWSensor");
		TimerTW = millis();
		//Türe
		if(pcf8574.read(7)==HIGH)
		{
			//Serial.println("Pin 8 is HIGH!");
			TuerOffen=1;			
		} 
		else
		{
			//Serial.println("Pin 8 is LOW!");
			TuerOffen=0;
		} 
		
		if (TuerOffen != TuerOffen_alt)
		{
			TuerFlag();
			TuerOffen_alt = TuerOffen;
			if (TuerOffen==1)
			{
				//61s lang darf die Tür offen sein
				TimerTuerOffen = millis();
			}
		}
		
		//Wasser
		if(pcf8574.read(0)==HIGH)
		{
			//Serial.println("Pin 1 is HIGH!");
			WasserLeer=1;
		} 
		else
		{
			//Serial.println("Pin 1 is LOW!");
			WasserLeer=0;
			//Status_Dampf0
			//Dampf = 0;
		} 
		if (WasserLeer != WasserLeer_alt)
		{
			WasserFlag();
			WasserLeer_alt =WasserLeer;
		}
	}
}
// ***************************************************************************************
// MQTT-Daten Senden
// ***************************************************************************************
void MQTT_Send()
{
	MQTT_Client.publish("DaniSauna", "online");
	Serial.println("MQTT-Publish");

	dtostrf(Status, 6, 0, arTXT);
	MQTT_Client.publish("DaniSauna/Status", arTXT);
	dtostrf(DampfSoll, 6, 0, arTXT);
	MQTT_Client.publish("DaniSauna/stat/DampfSoll", arTXT);
	dtostrf(OfenSoll, 6, 0, arTXT);
	MQTT_Client.publish("DaniSauna/stat/OfenSoll", arTXT);
	dtostrf(fTempIst, 6, 2, arTXT);
	MQTT_Client.publish("DaniSauna/fTempIst", arTXT);
	dtostrf(fDampfIst, 6, 2, arTXT);
	MQTT_Client.publish("DaniSauna/fDampfIst", arTXT);
	dtostrf(fTemp0, 6, 2, arTXT);
	MQTT_Client.publish("DaniSauna/fTemp0", arTXT);
	dtostrf(fTemp1, 6, 2, arTXT);
	MQTT_Client.publish("DaniSauna/fTemp1", arTXT);
	dtostrf(fTemp2, 6, 2, arTXT);
	MQTT_Client.publish("DaniSauna/fTemp2", arTXT);
	dtostrf(fTemp3, 6, 2, arTXT);
	MQTT_Client.publish("DaniSauna/fTemp3", arTXT);
	sprintf(arTXT, "%02d:%02d", gSettings.vhSet, gSettings.vmSet);
	MQTT_Client.publish("DaniSauna/stat/MaxDauer", arTXT);
	sprintf(arTXT, "%02d:%02d %02d.%02d", gSettings.vhh, gSettings.vmm,gSettings.vDD,gSettings.vMM);
	MQTT_Client.publish("DaniSauna/stat/Einschalt", arTXT);
	if (TuerOffen > 0)
	{
		MQTT_Client.publish("DaniSauna/TuerOffen", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/TuerOffen", "false");
	}
	if (WasserLeer > 0)
	{
		MQTT_Client.publish("DaniSauna/WasserLeer", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/WasserLeer", "false");
	}
	if (Ofen > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Ofen", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Ofen", "false");

	}
	if (Dampf > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Dampf", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Dampf", "false");
	}
	if (Ueberhitzung > 0)
	{
		MQTT_Client.publish("DaniSauna/Ueberhitzung", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/Ueberhitzung", "false");
	}
	if (gSettings.Termin > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Termin", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Termin", "false");
	}
	if (gSettings.Licht0 > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Licht0", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Licht0", "false");
	}
	if (gSettings.Licht1 > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Licht1", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Licht1", "false");
	}
	if (gSettings.Licht2 > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Licht2", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Licht2", "false");
	}
	// Licht-Status
	if (L0Status > 0)
	{
		MQTT_Client.publish("DaniSauna/L0Status", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/L0Status", "false");
	}
	if (L1Status > 0)
	{
		MQTT_Client.publish("DaniSauna/L1Status", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/L1Status", "false");
	}
	if (L2Status > 0)
	{
		MQTT_Client.publish("DaniSauna/L2Status", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/L2Status", "false");
	}
}
// ***************************************************************************************
// MQTT-Init-Daten Senden
// ***************************************************************************************
void MQTT_Send_Init()
{
	MQTT_Client.publish("DaniSauna", "online");
	Serial.println("MQTT-Init");

	dtostrf(Init_DampfSoll, 6, 0, arTXT);
	MQTT_Client.publish("DaniSauna/stat/DampfSoll", arTXT);
	dtostrf(Init_OfenSoll, 6, 0, arTXT);
	MQTT_Client.publish("DaniSauna/stat/OfenSoll", arTXT);
	MQTT_Client.publish("DaniSauna/stat/MaxDauer", Init_MaxDauer);
	MQTT_Client.publish("DaniSauna/stat/Einschalt", Init_Einschalt);
	MQTT_Client.publish("DaniSauna/stat/Reboot", "false");
	if (Init_Ofen > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Ofen", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Ofen", "false");

	}
	if (Init_Dampf > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Dampf", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Dampf", "false");
	}
	if (Init_Termin > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Termin", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Termin", "false");
	}
	if (Init_Licht0 > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Licht0", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Licht0", "false");
	}
	if (Init_Licht1 > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Licht1", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Licht1", "false");
	}
	if (Init_Licht2 > 0)
	{
		MQTT_Client.publish("DaniSauna/stat/Licht2", "true");
	}
	else
	{
		MQTT_Client.publish("DaniSauna/stat/Licht2", "false");
	}
	String ipAdressn =  GetWifiIp().toString();
	MQTT_Client.publish("DaniSauna/IP",ipAdressn.c_str());
}
// ***************************************************************************************
// MQTT-Loop
// ***************************************************************************************
void MQTT_Loop()
{
	if(gSettings.wifiOn)
	{
		// Nur MQTT aufrufen wenn WiFi connected ist
		if (WiFi.status() == WL_CONNECTED) 
		{
			// MQTT-Server-Loop
			if (!MQTT_Client.connected()) 
			{
				long now = millis();
				if (now - lastReconnectAttempt > 5000) {
					lastReconnectAttempt = now;
					// Attempt to reconnect
					if (MQTT_reconnect()) {
						lastReconnectAttempt = 0;
					}
				}
			}
			else
			{
				MQTT_Client.loop();
				if ((unsigned long) (millis() - TimerMQTT) >= intervallMQTT)
				{
					TimerMQTT = millis();
					if (Init_MQTT  < 10) //Nur beim ersten Mal die Initdaten senden
					{
						MQTT_Send_Init();
						OfenAus();
						Ofen = 0;
						Dampf = 0;	
						Tastenwechsel = 1;					
						Init_MQTT += 1;
					}
					else
					{
						MQTT_Send();
					}
				}
			}
		}	
	}
}
// ***************************************************************************************
// MQTT-Jetzt Senden
// ***************************************************************************************
void MQTT_Jetzt()
{
	if(gSettings.wifiOn)
	{
		// Nur MQTT aufrufen wenn WiFi connected ist
		if (WiFi.status() == WL_CONNECTED) 
		{
			// MQTT-Server-Loop
			if (!MQTT_Client.connected()) 
			{
				long now = millis();
				if (now - lastReconnectAttempt > 5000) {
					lastReconnectAttempt = now;
					// Attempt to reconnect
					if (MQTT_reconnect()) {
						lastReconnectAttempt = 0;
					}
				}
			}
			else
			{
				if (Init_MQTT < 10) //Nur beim ersten Mal die Initdaten senden
				{
					MQTT_Send_Init();
					OfenAus();
					Ofen = 0;
					Dampf = 0;
					Tastenwechsel = 1;
					Init_MQTT += 1;
				}
				else
				{
					MQTT_Send();
				}
			}
		}	
	}
}
// ***************************************************************************************
// Setup
// ***************************************************************************************
void setup() 
{
	InitFehler = false;
	Serial.begin(115200);
	DebugPrintLn();
	PCFInit();
	DisableWifi();
	if(LoadGlobalSettings())
	{
		kInfo = "Eeprom Daten OK\\r";
	}
	else
	{
		memset(&gSettings, 0, sizeof(gSettings));
		gSettings.version = version;
		sprintf(gSettings.hostname, "DanisSauna");
		sprintf(gSettings.apSsid, "MyWifi");
		gSettings.apChannel = 11;
		gSettings.apOn = false;
		gSettings.wifiOn = true;
		StoreGlobalSettings();
		kInfo = "Eeprom Daten neu angelegt\\r";	
	}
	SetHostname();
	
	if(gSettings.wifiOn)
	{
		if(StartWifi())
		{
			  kInfo += "WiFi verbunden ";
			  kInfo += GetWifiIp().toString();
			  kInfo += "\\r";

		}
	}
	else
	{
		kInfo += "Wifi aus\\r";
	}
	if(gSettings.apOn)
	{
		if(StartAp())
		{
			  //DebugPrintLn("AP started")
			  //DebugPrintLn(GetSoftApIp())
			  kInfo += "AP gestartet";
			  kInfo += GetSoftApIp().toString();
			  kInfo += "\\r";
		}
	}
	else
	{
		kInfo += "AP aus\\r";
	}
	delay(2500); //Nextion Display muß fertig sein
	InitializeNextionUi();
	delay(500);
	StartKonsole(kInfo);	
	CC2Init();
	StartKonsole(kInfo);	
	ADSInit();
	StartKonsole(kInfo);	
	InitRTC();
	StartKonsole(kInfo);	
	Datum().toCharArray(cDatum, Datum().length() + 1);
	kInfo += Datum();
	kInfo += "\\r";
	StartKonsole(kInfo);	

	FirmwareUpdate();

	MQTT_Client.setServer("mqtt-server.de", MQTT_port);
	MQTT_Client.setCallback(MQTT_Callback);
	if (InitFehler)
	{
		kInfo += "Abbruch wegen InitFehler\\r";
		kInfo += "Neustart in 5 Sekunden\\r";
		StartKonsole(kInfo);
		delay(5000);
		abort();
		//Ende des Programms Sauna werd nicht steuerbar
	}
	
	TimerRTCset = (unsigned long) millis() + 300000;
	NextionReadyCallback();
	Werte_Initialisieren();
	Soll_Eintragen();
	//Bildschirm Abschließen
	kInfo += "\\r";
	DebugPrintLn("Los gehts !!!");
	String kInfo_alt = kInfo;
	kInfo += "Froehliches Schwitzen !!!";
	//Text soll blinken ...
	for (size_t i = 0; i < 6; i++)
	{
		StartKonsole(kInfo);
		delay (330);
		StartKonsole(kInfo_alt);
		delay (330);
	}
	//Alles fertig, Bedienbildschirm zeigen
	LosGehts();
} 
// ***************************************************************************************
// Main LOOP
// ***************************************************************************************
void loop() 
{
	// OTA ermöglichen
	//ArduinoOTA.handle();
	//MQTT Schleife
	MQTT_Loop();
	// handle Nextion events
	HandleNextionEvents();
	// Messen und Schalten
	MessTemps();
	TuWSensor();
	if (TuerOffen == 2)
	{
		//Serial.println("Tuer-Update");
		//todo Nextion nachziehen !!!
		if (gSettings.Termin == 1)
		{
			gSettings.Termin = 0; //Fernsteuern aus !!!
			TerminNeu();
		}

		if ((unsigned long) (millis() - TimerTuerOffen) >= intervallTuerOffen)
		{			
			if (Ofen ==1)
			{
				OfenNeu(); //Ofen umschalten auf aus
				Ofen  = 0;
				Dampf = 0;
				Status = State_Aus;	
				for (size_t j = 1; j < 5; j++)
				{
					for (size_t i = 1; i < 4; i++)
					{
						pcf8574.write(i,!gSettings.Licht1 );
					}
					delay(200);
					for (size_t i = 1; i < 4; i++)
					{
						pcf8574.write(i,gSettings.Licht1 ); 
					}
					delay(200);
				}
				TimerTuerOffen = millis(); //Blinken zurücksetzen
			}
		}
	}
	// am Display getippt
	if (Tastenwechsel == 1)
	{
		Tastenwechsel = 0;
		//Serial.println("Tasten-Update + MQTT");
		Licht_Schalten();
		TempRegelung();
		MQTT_Jetzt();
	}
	
	// Tempregelung
	if ((unsigned long) (millis() - TimerPolling) >= intervallPolling)
	{
		//Serial.println("Temp-Regelung");
		TimerPolling = millis();
		TempRegelung();
	}
	// SSRs Schalten
	Status_Schalten();
		
	// RTC auf NTP synchronisieren
	if ((unsigned long) (millis() - TimerRTCset) >= intervallRTCset)
	{
		//Serial.println("RTC Update");
		TimerRTCset = millis();
		RTC_Set();
	}	

	// Zeit anzeigen, aktualisieren
	uint32 tic=millis();
	if(((prevPage != GetCurrentPage()) && GetCurrentPage()<5)||
	(((lastimeUpdate>tic && (UINT32_MAX-lastimeUpdate + tic)>timeUpdateInterval)|| 
	(lastimeUpdate<tic && (tic-lastimeUpdate)>timeUpdateInterval))&&
	GetCurrentPage()<5))
	{
		lastimeUpdate=tic;
		//Serial.println("Uhrzeit Update");
		DateTime t = rtc.now();
		timeUpdateInterval = (61 - t.second()) * 1000;
		if (timeUpdateInterval > 60000)
		{
			timeUpdateInterval = 35000;
		}
		UpdateTime(Zeit().c_str(), Datum().c_str(), t.unixtime());
		// Serial.println(DatumZeit());
		//Serial.print("UnixTime");Serial.println(t.unixtime());
	}
	
	// Nach 18s Temps updaten Wenn Seite 0
	if (GetCurrentPage()==1 && (unsigned long) (millis() - TimerShowTemp) >= intervallShowTemp)
	{
 		//Serial.println("frische Temps anzeigen");
    	TimerShowTemp = millis();
		ShowTemps();
 	}

	// WifiDaten anzeigen
	if((prevPage!=4 && GetCurrentPage()==4) ||
	(GetCurrentPage()==4 && 
	((lastInfoUpdate>tic && (UINT32_MAX-lastInfoUpdate + tic)>InfoUpdateInterval)|| 
	(lastInfoUpdate<tic && (tic-lastInfoUpdate)>InfoUpdateInterval))))
	{
		//Serial.println("Wifi-Update");
		lastInfoUpdate = tic;
		String info;
		GetWifiInfo(info);
		UpdateWifiInfo(info);
	}
	
	//Serial.print("PAGE :");Serial.println(GetCurrentPage());
	prevPage = GetCurrentPage();
}
