/**
 * @file nexionUi.cpp
 *
 * Nextion communication & objects definitions
 *
 * @author Jyrki Berg 2/17/2019 (https://github.com/jyberg)
 * 
 * @copyright 
 * Copyright (C) 2019 Jyrki Berg 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

#include "debugPrint.h"
#include "globalSettings.h"
#include <Nextion.h>
#include "wifi.h"
#include "nextionUi.h"
#include <NTPtimeESP.h>
#include <Time.h>

using namespace wifi;

namespace nextionUi
{
uint8_t currentPageId{0};

void InitializeNextionUi()
{
	Serial.println("Netxtion Init");
	if(nexInit(115200))
	{
		 Serial.println("Netxtion OK");
	}
	else
	{
		 Serial.println("Netxtion FAIL");
	}
	InitNextionCallbacks();
}

enum wifiPageType
{
	softAp=1,
	wifi=2
};
// page 0 start
NexPage pStart(0,0,"start");
NexText tKonsoleTxt(0,2,"konsole");

// page 1 home
NexPage pHome(1, 0, "home");

NexText tHomeTime(1,1,"time");
NexVariable bLampe0(1,4,"lampe0");
NexVariable bLampe1(1,5,"iampe1");
NexVariable bLampe2(1,6,"lampe2");
NexVariable bOfen(1,7,"ofen", &pHome);
NexVariable bDampf(1,8,"dampf", &pHome);
NexVariable vTempSollSet(1,9,"tempsollset", &pHome);
NexVariable vDampfSollSet(1,10,"feuchtsollset", &pHome);
NexVariable vTempSoll(1,11,"tempsoll", &pHome);
NexVariable vDampfSoll(1,12,"feuchtsoll", &pHome);
NexText tTempIst(1,15,"tempist");
NexText tOfenIst(1,17,"tempofen");
NexText tDampfIst(1,16,"feuchtist");
NexPicture vHomeSettings(1, 20,"b3");
NexText tRestZeit(1,26,"RestZeit");
NexVariable tCntDown(1,37,"tcntdown");

// page 2 home2
NexPage pHome2(2, 0, "home2");
NexTouch bb0(2,2,"b0");
NexTouch bb2(2,4,"b2");
NexTouch bb3(2,5,"b3");
NexPicture pHome2Settings(2, 5, "b3");
NexText tSetAusZeit(2,6,"tSetAusZeit");
NexVariable NhSet(2,8,"hSet", &pHome2);
NexVariable NmSet(2,9,"mSet", &pHome2);
NexVariable bTermin(2,17,"termin", &pHome2);
NexText tSetDatum(2,18,"tSetDatum");
NexVariable NDD(2,22,"vDD", &pHome2);
NexVariable NMM(2,23,"vMM", &pHome2);
NexVariable Nhh(2,24,"vhh", &pHome2);
NexVariable Nmm(2,25,"vmm", &pHome2);
NexText tSetUhr(2,27,"tSetUhr");
NexVariable NYYYY(2,32,"vYYYY", &pHome2);

// page 3 home3 / time
NexPage pHome3(3, 0, "home3");
NexPicture pHome3Settings(3, 6, "b3");
NexText tHome3Time(3,1,"time");
NexVariable gTextColour(3,2,"timeColour", &pHome3);
NexText tdatum(3,7,"datum");


// page 4 wifiInfo
NexPage pWifiInfo(4, 0, "wifiInfo");
NexPicture pWifiInfoSettings(4, 8, "settings");
NexText tWifiInfoTime(4,3,"t0"); //not needed because time is not global
NexText tWifiInfoTxt(4,2,"wifiInfoTxt");

// page 5 wifi
NexPage pWifi(5,0,"wifi");
NexTouch bWifiEnter(5,37,"enter");
NexText gSsid(5,39,"ssid", &pWifi);
NexText gPwd(5,41,"pwd", &pWifi);
NexVariable gWifiType(5,42,"type", &pWifi);
NexVariable gApHide(5,44,"hidden", &pWifi);
NexVariable gApChannel(5,48,"channelNro", &pWifi);

//page 6 system password
NexPage pSysPwd(6, 0, "password");
NexTouch bSysEnter(6, 38, "enter");
NexText txtSysPwd(6, 40, "inputTxt");

//page 7 settigs
NexPage pSettings(7, 0, "settings");

NexTouch bApOnOff(7,3,"m0");
NexTouch bWifiOnOff(7,4,"m1");
NexTouch bTimeZoneHour(7,6,"sHour");
NexTouch bApEdit(7,11,"apEdit");
NexTouch bWifiEdit(7,12,"wifiEdit");

NexVariable gApOn(7,1,"apOn", &pSettings);
NexVariable gWifiOn(7,2,"wifiOn", &pSettings);
NexVariable gZoneHour(7,8,"hour", &pSettings);
NexText gHostname(7,13,"hostname", &pSettings);

//page 8 hostname
NexPage pHostname(8, 0, "hostname");
NexTouch bEnterHostname(8,38,"enter");

//page 9 systemPwdChange
NexPage pChangePwd(9,0,"systemPwd");
NexTouch bChangePwdEnter(9, 37, "enter");
NexText gNewPwd(9,39,"pwd1",&pChangePwd);

//  event listener
NexTouch *nex_listen_list[] = {
	&vHomeSettings,
	&pHome2Settings,
	&pHome3Settings,
	&tHome3Time,
	&pWifiInfoSettings,
	&txtSysPwd,
	&bSysEnter,
	&bApOnOff,
	&bWifiOnOff,
	&bTimeZoneHour,
	&bEnterHostname,
	&bApEdit,
	&bWifiEdit,
	&bWifiEnter,
	&bChangePwdEnter,
	&bLampe0,
	&bLampe1,
	&bLampe2,
	&bOfen,
	&bDampf,
	&vTempSollSet,
	&vDampfSollSet,
	&bb0,
	&bb2,
	&bb3,
	&bTermin,
	nullptr
};

void InitNextionCallbacks()
{
	// functional callbacks
	currentPageIdCallback = currentPageCallback;
	nextionReadyCallback = NextionReadyCallback;

	// object callbacks
	vHomeSettings.attachPush(bHomeSettingsCallback);
	pHome2Settings.attachPush(bHomeSettingsCallback);
	pHome3Settings.attachPush(bHomeSettingsCallback);
	pWifiInfoSettings.attachPush(bHomeSettingsCallback);
	tHome3Time.attachPop(tHome3TimeCallback);
	bSysEnter.attachPush(bSysEnterCallback);
	bApOnOff.attachPop(bApOnOffCallback);
	bWifiOnOff.attachPop(bWifiOnOffCallback);
	bTimeZoneHour.attachPop(bTimeZoneHourCallback);
	bEnterHostname.attachPush(bEnterHostnameCallback);
	bApEdit.attachPush(bApEditCallback);
	bWifiEdit.attachPush(bWifiEditCallback);
	bWifiEnter.attachPush(bWifiEnterCallback);
	bChangePwdEnter.attachPush(bChangePwdEnterCallback);
	bLampe0.attachPush(bLampe0Callback);
	bLampe1.attachPush(bLampe1Callback);
	bLampe2.attachPush(bLampe2Callback);
	bLampe0.attachPop(bLampe0Callback);
	bLampe1.attachPop(bLampe1Callback);
	bLampe2.attachPop(bLampe2Callback);
	bOfen.attachPush(bOfenCallback);
	bDampf.attachPush(bDampfCallback);
	vTempSollSet.attachPop(vTempSollSetCallback);
	vDampfSollSet.attachPop(vDampfSollSetCallback);
	bb0.attachPush(bb1callback);
	bb2.attachPush(bb2callback);
	bb3.attachPush(bb3callback);
	bTermin.attachPush(bTerminCallback);
}
// Nextion callback functions
void bLampe0Callback (void *ptr)
{
	int32_t val{-1};
	for(int i{0}; i < 100; ++i)
	{
		bLampe0.getValue(&val);
		if(val == 1)
		{
			gSettings.Licht0 = 1; //Ein
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
		if(val == 0)
		{
			gSettings.Licht0 = 0; //Aus
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
		
	}
}
void bLampe1Callback (void *ptr)
{
	int32_t val{-1};
	for(int i{0}; i < 100; ++i)
	{
		bLampe1.getValue(&val);
		if(val == 1)
		{
			gSettings.Licht1 = 1; //Ein
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
		if(val == 0)
		{
			gSettings.Licht1 = 0; //Aus
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
		
	}
}
void bLampe2Callback (void *ptr){
	int32_t val{-1};
	for(int i{0}; i < 100; ++i)
	{
		bLampe2.getValue(&val);
		if(val == 1)
		{
			gSettings.Licht2 = 1; //Ein
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
		if(val == 0)
		{
			gSettings.Licht2 = 0; //Aus
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
		
	}
}
void bOfenCallback (void *ptr){
	int32_t val{-1};
	for(int i{0}; i < 15; ++i)
	{
		bOfen.getValue(&val);
		Serial.print("Ofen: "); Serial.println(val);
		if(val == 1)
		{
			Ofen = 1; //Ein
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
		if(val == 0)
		{
			Ofen = 0; //Aus
			Dampf = 0;
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
	}
}
void bDampfCallback (void *ptr){
	int32_t val{-1};
	for(int i{0}; i < 15; ++i)
	{
		bDampf.getValue(&val);
		Serial.print("Dampf: "); Serial.println(val);
		if(val == 1)
		{
			Dampf = 1; //Ein
			Ofen = 1;
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
		if(val == 0)
		{
			Dampf = 0; //Aus
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
	}
}
void vTempSollSetCallback (void *ptr){
	int32_t val{-1};
	for(int i{0}; i < 15; ++i)
	{
		vTempSollSet.getValue(&val);
		Serial.print("OfenSoll: "); Serial.println(val);
		if (val > 39)
		{			
			OfenSoll= val;
			gSettings.vTempSollSet=val;
			Serial.print("OfenSoll: "); Serial.println(val);
			Tastenwechsel = 1;
			StoreGlobalSettings();
			Serial.println("OfenSoll speichern "); 			
			break;
		}
	}
}
void vDampfSollSetCallback (void *ptr){
	int32_t val{-1};
	for(int i{0}; i < 15; ++i)
	{
		vDampfSollSet.getValue(&val);
		Serial.print("DampfSoll: "); Serial.println(val);
		if (val > 9)
		{
			DampfSoll= val;
			gSettings.vDampfSollSet=val;
			Serial.print("DampfSoll: "); Serial.println(val);
			Tastenwechsel = 1;
			StoreGlobalSettings();
			Serial.println("DampfSoll speichern "); 
			break;
		}
	}
}
void bHomeSettingsCallback(void *ptr)
{
	currentPageId=255;
	if(gSettings.systemPwd[0] == 0)
	{
		ShowSystemSettings();
	}
	else
	{
		pSysPwd.show();
	}
}
void bSysEnterCallback(void *ptr)
{
	String pwd;
	txtSysPwd.getText(pwd);
	if(ValidatePassword(pwd))
	{
		ShowSystemSettings();
	}
	else
	{
		txtSysPwd.setText("");
	}
}
void bApOnOffCallback(void *ptr)
{
	int32_t val{-1};
	for(int i{0}; i < 5; ++i)
	{
		gApOn.getValue(&val);
		if(val!=2)
		{
			gSettings.apOn=val?true:false;
			if(val)
			{
				wifi::StartAp();
			}
			else
			{
				wifi::StopAp();
			}
			StoreGlobalSettings();
		}
	}
}
void bWifiOnOffCallback(void *ptr)
{
	int32_t val{-1};
	for(int i{0}; i < 5; ++i)
	{
		gWifiOn.getValue(&val);
		if(val!=2)
		{
			gSettings.wifiOn=val?true:false;
			if(val)
			{
				wifi::StartWifi();
			}
			else
			{
				wifi::StopWifi();
			}
			StoreGlobalSettings();
		}
	}
}
void bTimeZoneHourCallback(void *ptr)
{
	int32_t val{-1};
	if(gZoneHour.getValue(&val))
	{
		gSettings.timeZoneHour=val;
		StoreGlobalSettings();
		configTime(gSettings.timeZoneHour*60*60, 0, "ch.pool.ntp.org", "pool.ntp.org");
	}
}
void bEnterHostnameCallback(void *ptr)
{
	delay(50);
	String str;
	if(gHostname.getText(str))
	{
		uint16_t strLen = str.length()+1;
		strncpy(gSettings.hostname,str.c_str(),strLen>sizeof(gSettings.hostname)?sizeof(gSettings.hostname):strLen);
		gSettings.hostname[sizeof(gSettings.hostname)-1]=0;
		StoreGlobalSettings();
		SetHostname();
	}
}
void bApEditCallback(void *ptr)
{
	gWifiType.setValue(wifiPageType::softAp);
	gSsid.setText(gSettings.apSsid);
	gPwd.setText(gSettings.apPwd);
	gApHide.setValue(gSettings.apHide?1:0);
	gApChannel.setValue(gSettings.apChannel);
	pWifi.show();
}
void bWifiEditCallback(void *ptr)
{
	gWifiType.setValue(wifiPageType::wifi);
	Serial.print("set NRO = "); Serial.println(wifiPageType::wifi);
	gSsid.setText(gSettings.wlanSsid);
	gPwd.setText(gSettings.wlanPwd);
	pWifi.show();
}
void bWifiEnterCallback(void *ptr)
{
	int32_t nro{-1};
	String str;
	gWifiType.getValue(&nro);
	//Serial.println("Juwel editiert");
	if(gWifiType.getValue(&nro))
	{
		//Serial.print("get NRO = "); Serial.println(nro);
		if(nro == wifiPageType::softAp)
		{
			gSsid.getText(str);
			if(gSsid.getText(str))
			{
				uint16_t strLen = str.length()+1;
				strncpy(gSettings.apSsid,str.c_str(),strLen>sizeof(gSettings.apSsid)?sizeof(gSettings.apSsid):strLen);
				gSettings.apSsid[sizeof(gSettings.apSsid)-1]=0;
			}
			gPwd.getText(str);
			if(gPwd.getText(str))
			{
				uint16_t strLen = str.length()+1;
				strncpy(gSettings.apPwd,str.c_str(),strLen>sizeof(gSettings.apPwd)?sizeof(gSettings.apPwd):strLen);
				gSettings.apPwd[sizeof(gSettings.apPwd)-1]=0;	
			}
			if(gApChannel.getValue(&nro))
			{
				gSettings.apChannel=nro;
			}
			if(gApHide.getValue(&nro))
			{
				gSettings.apHide=nro?true:false;
			}
		}
		else if(nro == wifiPageType::wifi)
		{
			gSsid.getText(str);
			if(gSsid.getText(str))
			{
				Serial.print("SSID: ");Serial.println(str);
				uint16_t strLen = str.length()+1;
				strncpy(gSettings.wlanSsid,str.c_str(),strLen>sizeof(gSettings.wlanSsid)?sizeof(gSettings.wlanSsid):strLen);
				gSettings.wlanSsid[sizeof(gSettings.wlanSsid)-1]=0;
				Serial.print("SSID: ");Serial.println(gSettings.wlanSsid);
			}
			gPwd.getText(str);
			if(gPwd.getText(str))
			{
				Serial.print("PWD: ");Serial.println(str);
				uint16_t strLen = str.length()+1;
				strncpy(gSettings.wlanPwd,str.c_str(),strLen>sizeof(gSettings.wlanPwd)?sizeof(gSettings.wlanPwd):strLen);
				gSettings.wlanPwd[sizeof(gSettings.wlanPwd)-1]=0;	
				Serial.print("PWd: ");Serial.println(gSettings.wlanPwd);
			}
		}
		StoreGlobalSettings();
		StoreGlobalSettings();
		StopAp();
		StopWifi();
		if(gSettings.wifiOn)
		{
			StartWifi();
		}	
		if(gSettings.apOn)
		{
			StartAp();
		}	
	}
}
void bChangePwdEnterCallback(void *ptr)
{
	String str;
	if(gNewPwd.getText(str))
	{
		uint16_t strLen = str.length()+1;
		strncpy(gSettings.systemPwd,str.c_str(),strLen>sizeof(gSettings.systemPwd)?sizeof(gSettings.systemPwd):strLen);
		gSettings.systemPwd[sizeof(gSettings.systemPwd)-1]=0;
		StoreGlobalSettings();
	}
}
void tHome3TimeCallback(void *ptr)
{
	int32_t nro{-1};
	for(int i{0}; i < 8; ++i)
	{
		gTextColour.getValue(&nro);
		if(nro > -1)
		{
			gSettings.textColour=nro;
			//Serial.print("TextFarbe:"); Serial.println(nro);
			StoreGlobalSettings();
			break;
		}
	}
}
void currentPageCallback(uint8_t pageId)
{
	currentPageId=pageId;
}
void ShowSystemSettings()
{
	gHostname.setText(gSettings.hostname);
	gApOn.setValue(gSettings.apOn?1:0);
	gWifiOn.setValue(gSettings.wifiOn?1:0);
	gZoneHour.setValue(gSettings.timeZoneHour);
	pSettings.show();
}
void ShowTemps()
{
	tTempIst.setText(TempIst);
	tDampfIst.setText(DampfIst);
	tOfenIst.setText(OfenIst);
}
void Soll_Eintragen()
{
	// Serial.println("Soll_eintragen");
	Lampe0Neu();
	Lampe1Neu();
	Lampe2Neu();
	// Serial.println("Lampen fertig");
	DampfSollNeu();
	OfenSollNeu();
	// Serial.println("Ofen+Dampf fertig");
	TerminNeu();
	// Serial.println("Termin fertig");
	LaufzeitNeu();
	StartZeitNeu();
	// Serial.println("Zeiten fertig");
	TuerFlag();
	WasserFlag();
	tdatum.setText(cDatum);
}
// Sichtbarkeiten();
void Sichtbarkeiten()
{
	String cmd = String("vis tempsollset,0");
    sendCommand(cmd.c_str());
	cmd = String("vis feuchtsollset,0");
    sendCommand(cmd.c_str());
	cmd = String("vis tempsoll,0");
    sendCommand(cmd.c_str());
	cmd = String("vis feuchtsoll,0");
    sendCommand(cmd.c_str());
}
void Status_Polling()
{
	int32_t val{-1};
	//Verdampfer an?
	for(int i{0}; i < 100; ++i)
	{
		bDampf.getValue(&val);
		if(val == 1)
		{
			Dampf = 1; //Ein
			Tastenwechsel = 1;
			break;
		}
		if(val == 0)
		{
			Dampf = 0; //Aus
			Tastenwechsel = 1;
			break;
		}
	}
	//Dampfsoll ?
	for(int i{0}; i < 5; ++i)
	{
		vDampfSollSet.getValue(&val);
		if (val > 9)
		{
			DampfSoll= val;
			//Serial.print("DampfSoll: "); Serial.println(val);
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}

	}
	//Saunaofen an?
	for(int i{0}; i < 100; ++i)
	{
		bOfen.getValue(&val);
		if(val == 1)
		{
			Ofen = 1; //Ein
			Tastenwechsel = 1;
			break;
		}
		if(val == 0)
		{
			Ofen = 0; //Aus
			Tastenwechsel = 1;
			break;
		}
	}
	//dann Soll ermitteln
	for(int j{0}; j < 5; ++j)
	{
		vTempSollSet.getValue(&val);
		if (val > 39)
		{			
			OfenSoll= val;
			//Serial.print("OfenSoll: "); Serial.println(val);
			Tastenwechsel = 1;
			StoreGlobalSettings();
			break;
		}
	}

	for(int i{0}; i < 100; ++i)
	{
		bLampe0.getValue(&val);
		if(val == 1)
		{
			gSettings.Licht0 = 1; //Ein
			Tastenwechsel = 1;

			break;
		}
		if(val == 0)
		{
			gSettings.Licht0 = 0; //Aus
			Tastenwechsel = 1;
			break;
		}
		
	}
	for(int i{0}; i < 100; ++i)
	{
		bLampe1.getValue(&val);
		if(val == 1)
		{
			gSettings.Licht1 = 1; //Ein
			Tastenwechsel = 1;

			break;
		}
		if(val == 0)
		{
			gSettings.Licht1 = 0; //Aus
			Tastenwechsel = 1;
			break;
		}
		
	}
	for(int i{0}; i < 100; ++i)
	{
		bLampe2.getValue(&val);
		if(val == 1)
		{
			gSettings.Licht2 = 1; //Ein
			Tastenwechsel = 1;

			break;
		}
		if(val == 0)
		{
			gSettings.Licht2 = 0; //Aus
			Tastenwechsel = 1;
			break;
		}
	}
}
void bb1callback(void *ptr)
{
	currentPageId = 1;
	bbxcallback();
}
void bb2callback(void *ptr)
{
	currentPageId = 3;
	bbxcallback();
}
void bb3callback(void *ptr)
{
	currentPageId = 4;
	bbxcallback();
}
void bbxcallback()
{
	int32_t val;
	val = -1;
	for(int i{0}; i < 18; ++i)
	{
		// Tagestimer
		Nmm.getValue(&val);
			//Serial.print("vmm: ");Serial.println(val);
		if ((val > -1) & (60 > val))
		{			
			gSettings.vmm = val;
			break;
		}
	}
	val = -1;
	for(int i{0}; i < 8; ++i)
	{
		// Tagestimer
		Nhh.getValue(&val);
			//Serial.print("vhh: ");Serial.println(val);
		if ((val > -1) & (24 > val))
		{			
			gSettings.vhh = val;
			break;
		}
	}
	val = -1;
	for(int i{0}; i < 8; ++i)
	{
		// Tagestimer
		NDD.getValue(&val);
			//Serial.print("vDD: ");Serial.println(val);
		if ((val > 0) & (32 > val))
		{			
			gSettings.vDD = val;
			break;
		}
	}
	val = -1;
	for(int i{0}; i < 8; ++i)
	{
		// Tagestimer
		NMM.getValue(&val);
			//Serial.print("vMM: ");Serial.println(val);
		if ((val > 0) & (13 > val))
		{			
			gSettings.vMM = val;
			break;
		}
	}
	val = -1;
	for(int i{0}; i < 8; ++i)
	{
		// Tagestimer
		NhSet.getValue(&val);
			//Serial.print("hSet: ");Serial.println(val);
		if ((val > -1) & (24 > val))
		{			
			gSettings.vhSet = val;
			break;
		}
	}
	val = -1;
	for(int i{0}; i < 8; ++i)
	{
		// Tagestimer
		NmSet.getValue(&val);
			//Serial.print("mSet: ");Serial.println(val);
		if ((val > 0) & (60 > val))
		{			
			gSettings.vmSet = val;
			break;
		}
	}
	Serial.println("Vxx Speichern");
	StoreGlobalSettings();
}
void bTerminCallback(void *ptr)
{
	int32_t val;
	val = -1;
	for(int i{0}; i < 18; ++i)
	{
		// Tagestimer
		bTermin.getValue(&val);
			Serial.print("bTermin: ");Serial.println(val);
		if ((val > -1) & (2 > val))
		{			
			gSettings.Termin = val;
			StoreGlobalSettings();
			Tastenwechsel = 1;
			break;
		}
	}
}
void TuerFlag()
{
	if (TuerOffen ==1)
	{
		String cmd = String("vis tTuer,1");
		sendCommand(cmd.c_str());
	}
	if (TuerOffen ==0)
	{
		String cmd = String("vis tTuer,0");
		sendCommand(cmd.c_str());
	}
}
void WasserFlag()
{
	if (WasserLeer ==1)
	{
		String cmd = String("vis tWasser,0");
		sendCommand(cmd.c_str());
	}
	if (WasserLeer ==0)
	{
		String cmd = String("vis tWasser,0");
		sendCommand(cmd.c_str());
	}
}
void OfenNeu()
{
	String cmd = String("click ofen,1");
	sendCommand(cmd.c_str());
}
void OfenAus()
{
	int32_t val{-1};
	for(int i{0}; i < 100; ++i)
	{
		bOfen.getValue(&val);
		if(val == 1)
		{
			String cmd = String("click ofen,1");
			sendCommand(cmd.c_str());
			break;
		}

	}	
}
void DampfNeu()
{
	String cmd = String("click dampf,1");
	sendCommand(cmd.c_str());
}
void DampfSollNeu()
{
	vDampfSoll.setValue(gSettings.vDampfSollSet);
	//Serial.print("DampfSoll: ");Serial.println(gSettings.vDampfSoll);
	vDampfSollSet.setValue(gSettings.vDampfSollSet);
	Serial.print("DampfSollSet: ");Serial.println(gSettings.vDampfSollSet);
}
void OfenSollNeu()
{
	vTempSoll.setValue(gSettings.vTempSollSet);
	//Serial.print("OfenSollSet: ");Serial.println(gSettings.vTempSoll);
	vTempSollSet.setValue(gSettings.vTempSollSet);
	Serial.print("vOfenSollSet: ");Serial.println(gSettings.vTempSollSet);	
}
void Lampe0Neu()
{
	bLampe0.setValue(gSettings.Licht0);
}
void Lampe1Neu()
{
	bLampe1.setValue(gSettings.Licht1);
}
void Lampe2Neu()
{
	bLampe2.setValue(gSettings.Licht2);
}
void TerminNeu()
{
	//Serial.print("Termin ist: ");Serial.println(gSettings.Termin);
	bTermin.setValue(gSettings.Termin);		
}
void LaufzeitNeu()
{	
	// Serial.print("vmm: ");Serial.println(gSettings.vmm);
	// Serial.print("vhh: ");Serial.println(gSettings.vhh);
	delay(50);
	Nmm.setValue(gSettings.vmm);
	Nhh.setValue(gSettings.vhh);
}
void StartZeitNeu()
{
	delay(50);
	NmSet.setValue(gSettings.vmSet);
	NhSet.setValue(gSettings.vhSet);
	delay(50);
	NDD.setValue(gSettings.vDD);
	NMM.setValue(gSettings.vMM);
	// Serial.print("vMM: ");Serial.println(gSettings.vMM);
	// Serial.print("vDD: ");Serial.println(gSettings.vDD);
	// Serial.print("NhSet: ");Serial.println(gSettings.vhSet);
	// Serial.print("NmSet: ");Serial.println(gSettings.vmSet);
}
void StartKonsole(String &info)
{
	tKonsoleTxt.setText(info.c_str());
}
void UpdateTime(const char* time, const char* tdate, unsigned long t )
{
	if(currentPageId<5)
	{
		tHomeTime.setText(time); // updates curren page time object
		tWifiInfoTime.setText(time);
		if (cDatum_alt != tdate)
		{
			tdatum.setText(tdate);
			NYYYY.setText(gSettings.vYYYY);
			if (gSettings.Termin == 1)
			{
				/* Berechnung der verbleibenden Tage:Stunden:Minuten */

				unsigned long  tDiff;
				tmElements_t tm;
				tm.Minute = gSettings.vmm;
				tm.Hour = gSettings.vhh;
				tm.Day = gSettings.vDD;
				tm.Month = gSettings.vMM;
				tm.Year = Jahr - 1970;
				time_t iCntDwn = makeTime(tm);
				tDiff = iCntDwn - t;
				//Tage Studen und Minuten berechnen
				unsigned long Tag;
				unsigned long Std;
				unsigned long Min;
				Tag = tDiff/86400;
				Std = (tDiff - Tag*86400)/3600;
				Min = (tDiff - Tag*86400 - Std*3600)/60;
				//Ausgabestring zusammenbauen
				char tTemp [11];
				String tSend;
				dtostrf(Tag, 1, 0, tTemp);
				tSend = tTemp;
				tSend +=":";
				if (Std<10)
				{
					tSend += "0";
				}
				dtostrf(Std, 1, 0, tTemp);
				tSend += tTemp;
				tSend +=":";
				if (Min<10)
				{
					tSend += "0";
				}
				dtostrf(Min, 1, 0, tTemp);
				tSend += tTemp ;
				tRestZeit.setText(tSend.c_str());
				/*
				Serial.print ("Text: "); Serial.println(tSend);				
				Serial.print ("gMinute: "); Serial.println(gSettings.vmm);
				Serial.print ("gHour: "); Serial.println(gSettings.vhh);
				Serial.print ("gDay: "); Serial.println(gSettings.vDD);
				Serial.print ("Diff: "); Serial.println(tDiff);
				Serial.print ("t: "); Serial.println(t);
				Serial.print ("StartZeit: "); Serial.println(iCntDwn);
				*/
			}
		}
	}
}
void UpdateWifiInfo(String &info)
{
	tWifiInfoTxt.setText(info.c_str());
}
uint8_t GetCurrentPage()
{
	return currentPageId;
}
void NextionReadyCallback()
{
	delay(50);
	gTextColour.setValue(gSettings.textColour);
	delay(50);
	//pStart.show();
}
void HandleNextionEvents()
{
 	// handle Nextion events
	nexLoop(nex_listen_list);
}
void LosGehts()
{
	currentPageId = 1;
	sendCommand("page 1");
}
}
